package frazioniconeccezioni;

public class Frazione {

	private int num, den;
	
	public Frazione( int num, int den ) throws DenominatoreNulloException {
		if ( den == 0 ) throw new DenominatoreNulloException();
		boolean negativo = num * den < 0;
		this.num = Math.abs( num );
		this.den = Math.abs( den );
		int m = gcd( this.num, this.den );
		this.num /= m;
		this.den /= m;
		if ( negativo ) this.num *= -1;
	}
	
	public static int gcd( int x, int y ) {
		int r;
		r = x % y;
		while ( r != 0 ) {
			x = y;
			y = r;
			r = x % y;
		}
		return y;
	}
	
	public Frazione piu( Frazione f ) {
		int den = this.den * f.den;
		int num = this.num * f.den + f.num * this.den;
		try {
			return new Frazione( num, den );
		}
		catch ( DenominatoreNulloException e ) {
			return null; // Cannot happen!
		}
	}
	
	public Frazione meno() {
		try {
			return new Frazione( -num, den );
		}
		catch ( DenominatoreNulloException e ) {
			return null; // Cannot happen!
		}
	}
	
	public Frazione meno( Frazione f ) {
		return piu( f.meno() );
	}
	
	public Frazione per( Frazione f ) {
		try {
			return new Frazione( num * f.num, den * f.den );
		}
		catch ( DenominatoreNulloException e ) {
			return null; // Cannot happen!
		}
	}
	
	public Frazione diviso() throws DenominatoreNulloException {
		return new Frazione( den, num );
	}
	
	public Frazione diviso( Frazione f ) throws DenominatoreNulloException {
		return per( f.diviso() );
	}
	
	public boolean isMinore( Frazione f ) {
		return this.meno( f ).num < 0;
	}
	
	public String toString() {
		if ( den == 1) return "" + num;
		return num + "/" + den;
	}
	
	public boolean equals( Object x ) {
		if ( x instanceof Frazione ) {
			return this.meno((Frazione) x ).num == 0;
		} else return false;
	}
	
	public int hashCode() {
		return num + den;
	}
	
}
